/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "probe.h"

/* ******************************************************************************************* */

int build_port_name(char *port_name,char *port_location,int port_number){
/* build the name of the port corresponding to the requested number */
/* return EXIT_SUCCESS or EXIT_FAILURE */
	int number;		/* multipurpose integer */
	
	/* checking that the required port number is within the accepted range */
	if(port_number>SERIAL_PORTS_MAX) {
		perror("build_port_name: asking for a port number > SERIAL_PORTS_MAX");
	}
	/* building the serial port file name */
	number=snprintf(port_name,(size_t)(FILE_NAME_LENGTH*sizeof(char)),"%s%d",port_location,port_number);
	if(number <= -1 || number >= (int)(FILE_NAME_LENGTH*sizeof(char))) {
		perror("build_port_name: port descriptor string building failure\n");
		return EXIT_FAILURE;
	}
	
	return EXIT_SUCCESS;
}

int start_probe(struct probe_descriptor *probe,int probe_index,struct configuration_struct *settings) {
/* open and initialize a port decsribed by its structure "probe_descriptor" */
/* return EXIT_SUCCESS or EXIT_FAILURE */
	char port_name[FILE_NAME_LENGTH];		/* the place where we will read the probe values */
	char log_file_name[FILE_NAME_LENGTH];		/* name of the log file */
	struct tm *local_time;				/* local time structure */
	char log_date[20];				/* string containing the ISO formated time */
	time_t timing;					/* time in seconds since EPOC */
	int n;
	
	/* initializing and run the probes */
	fprintf(std_msg,"\tInitializing probe #%d\n",probe_index);
	/* number of the probe we are talking about */
	probe->number=probe_index;
	/* probe has not yet recorded events */
	probe->value=0;
	/* probe location information */
	probe->location[0]='\0';
	strncat(probe->location,settings->generic_location,FILE_NAME_LENGTH);
	if(build_port_name(port_name,probe->location,probe->number) == EXIT_FAILURE) {
		return EXIT_FAILURE;
	}
	
	/* opening and initializing log file if needed */
	probe->log_file=NULL;
	if(settings->file_log==EXIT_SUCCESS) {
		/* building the file name of the probe log file */
		log_file_name[0]='\0';
		if(time(&timing) == ((time_t)-1)) {
			perror("start_probe: Time reading failure\n");
			return EXIT_FAILURE;
		}
		if((local_time=(struct tm*)localtime(&timing)) != NULL) { /* cast done because gcc was complaining. Why??? */
			strftime(log_date,(size_t)20,"%m%d%y",localtime(&timing));
		} else {
			perror("start_probe: Time conversion failure");
			return EXIT_FAILURE;
		}
		n=snprintf(log_file_name,(size_t)(FILE_NAME_LENGTH*sizeof(char)),"%s%d_%s.%s",PROBE_LOG_FILE,probe->number,log_date,PROBE_LOG_EXTENSION);
		if(n <= -1 || n >= ((int)(FILE_NAME_LENGTH*sizeof(char)))) {
			perror("start_probe: Log file name too long\n");
			return EXIT_FAILURE;
		}
		/* opening the probe log file */
		if((probe->log_file=fopen(log_file_name,"wt")) == NULL){
			perror("main: Log file can not be opened");
			return EXIT_FAILURE;
		}
		/* header for the probe log file */
		fprintf(probe->log_file,"#%s %s (%s) - %s\n",GM4LIN_NAME,GM4LIN_VERSION,GM4LIN_DATE,GM4LIN_COMPANY);
		if(settings->pulse_mode==EXIT_FAILURE) fprintf(probe->log_file,"#Radiation activity record\n#Date_Time cpm R/h(Co60)\n");
		else fprintf(probe->log_file,"#Radiation activity record\n#Date_Time raw_pulse R/h(Co60)\n");
	}
	
	/*************************************************/
	/* initialization of the port */
	//fprintf(std_msg,"tnPort \"%s\" will be opened\n",port_name);	
	if((probe->file_descriptor=open_port(port_name)) == EXIT_FAILURE) {
		perror("run_probe: port can not be opened");
		return EXIT_FAILURE;
	}
	if((set_port(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: port can not be set up");
		return EXIT_FAILURE;
	}
	if((dtr_up(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: DTR line of port can not be raised");
		return EXIT_FAILURE;
	}
		
	return EXIT_SUCCESS;
}

int stop_probe(struct probe_descriptor *probe) {
/* close port and shutdown the device */
/* return EXIT_SUCCESS or EXIT_FAILURE */
	/* end of probe acquisition, we close the port */
	if((dtr_down(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: DTR line of port can not be lowered");
	}
	if((close_port(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: port can not be closed");
		return EXIT_FAILURE;
	}
	
	return EXIT_SUCCESS;
}

int read_probe(struct probe_descriptor *probe) {
/* read the buffer of the selected probe and print it to its log file */
/* return EXIT_SUCCESS or EXIT_FAILURE */
	char *data_line;		/* formatted data extracted from the probe */
	char *iso_time;			/* string to store the ISO formatted time for the log file */
	
	/* allocating memory for the time string used by the event log */
	if((iso_time=(char *)calloc(20,sizeof(char)))== NULL){
		perror("run_probe: String memory allocation failure for \"iso_time\"");
		return EXIT_FAILURE;
	}
	/* allocating memory for the data line generated by the event log */
	if((data_line=(char *)calloc(DATA_LINE_LENGTH,sizeof(char)))== NULL){
		perror("run_probe: String memory allocation failure for \"data_line\"");
		return EXIT_FAILURE;
	}
	if((read_port(probe->file_descriptor,data_line,iso_time)) == EXIT_FAILURE) {
			perror("run_probe: reading failed, skipping");
		}
	//fprintf(std_msg,"\tData line read:%s",data_line);
	fprintf(probe->log_file,"%s\n",data_line);
	
	return EXIT_SUCCESS;
}

int run_probe(struct probe_descriptor *probe) {
/* start, initialize and read data from a port described by its probe_descriptor structure */
/* this function is for multi-thread purpose */
/* return EXIT_SUCCESS or EXIT_FAILURE */
	char *port_name;		/* the place where we will read the probe values */
	char *data_line;		/* formatted data extracted from the probe */
	char *iso_time;			/* string to store the ISO formatted time for the log file */
		
	/* allocating memory for the serial port file name */
	if((port_name=(char *)calloc(FILE_NAME_LENGTH,sizeof(char)))== NULL){
		perror("run_probe: String memory allocation failure for \"port_name\"");
		return EXIT_FAILURE;
	}
	/* allocating memory for the time string used by the event log */
	if((iso_time=(char *)calloc(20,sizeof(char)))== NULL){
		perror("run_probe: String memory allocation failure for \"iso_time\"");
		return EXIT_FAILURE;
	}
	/* allocating memory for the data line generated by the event log */
	if((data_line=(char *)calloc(DATA_LINE_LENGTH,sizeof(char)))== NULL){
		perror("run_probe: String memory allocation failure for \"data_line\"");
		return EXIT_FAILURE;
	}
	build_port_name(port_name,probe->location,probe->number);
	//fprintf(std_msg,"\tPort \"%s\" will be opened\n",port_name);
	
	/*******************************************************************************/
	
	/* initialization of the port */
	if((probe->file_descriptor=open_port(port_name)) == EXIT_FAILURE) {
		perror("run_probe: port can not be opened");
		return EXIT_FAILURE;
	}
	/*if((set_port(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: port can not be set up");
		return EXIT_FAILURE;
	}*/
	if((dtr_up(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: DTR line of port can not be raised");
		return EXIT_FAILURE;
	}
	
	/* reading data from the port */
	/* allocating memory for the timeout structure */
	while (flag_stop == FALSE) {
		if((read_port(probe->file_descriptor,data_line,iso_time)) == EXIT_FAILURE) {
			perror("run_probe: reading failed, skipping");
		}
		//fprintf(std_msg,"Data line read:%s",data_line);
		fprintf(probe->log_file,"%s\n",data_line);
	}
	
	/* end of probe acquisition, we close the port */
	if((dtr_down(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: DTR line of port can not be lowered");
	}
	if((close_port(probe->file_descriptor)) == EXIT_FAILURE) {
		perror("run_probe: port can not be closed");
		return EXIT_FAILURE;
	}
	/* it is time to free memory */
	free(port_name);
	free(data_line);
	free(iso_time);
	
	return EXIT_SUCCESS;
}


