/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef structures_H
#define structures_H
#include "/usr/include/mysql/mysql.h"
#include "my_limits.h"

/* this structure is used to store all the settings */
typedef struct configuration_struct {
	/* serial connection */
	char generic_location[FILE_NAME_LENGTH];		/* generic location of the probes */
	int num_probes;						/* number of probes too log */
	int probe_index_start;					/* index of the first probe (the "0" in /dev/ttyS0) */
	int file_descriptor_max;				/* maximum index of the opened file descriptors (could be > num_probes) */
	/* mode and sampling rates */
	int activity_data_write_sampling;			/* sampling for data log, activity mode */
	int pulse_data_write_sampling;				/* sampling for data log, pulse mode */
	int pulse_mode;						/* EXIT_SUCCESS if we are in radiation pulse mode */
	int pulse_particule_sub_sampling;			/* sub_sampling rate for pulse mode*/
	int pulse_mode_particules_sampling_count;		/* every ... particules sampling periods, we will write data to file */
	int pulse_mode_particules_sampling_write;		/* every ... particules sampling periods, we will write data to file */
	int file_log;						/* log data to a file (on per probe) */
	#ifndef SQL_LIBS
	/* sql connection */
	MYSQL mysql;						/* MySQL database pointer */
	int sql_log;						/* are we going to log data to an SQL server? */
	char sql_host[SQL_PARAMETERS_STRINGS];			/* if logging data to a MySQL server, host name */
	char sql_database[SQL_PARAMETERS_STRINGS];		/* if logging data to a MySQL server, database name */
	char sql_table[SQL_PARAMETERS_STRINGS];			/* if logging data to a MySQL server, table name */
	char sql_field[SQL_PARAMETERS_STRINGS];			/* if logging data to a MySQL server, field name */
	char sql_user[SQL_PARAMETERS_STRINGS];			/* if logging data to a MySQL server, user name */
	char sql_password[SQL_PARAMETERS_STRINGS];		/* if logging data to a MySQL server, password */
	#endif
} configuration_structure;

/* data needed for a probe acquisition */
typedef struct probe_descriptor {
	FILE *log_file;				/* log file associated with this probe */
	char location[FILE_NAME_LENGTH];	/* general location of the probe (like "/dev/ttyS") */
	int number;				/* probe number (like 0 -> it will be /dev/ttyS0) */
	int file_descriptor;			/* file_descriptor of the probe */
	int sensitivity;			/* sensitivity for the calibration of the probe (in cps/mR/h) */
	float value;				/* number of events recorded by this probe since ... last time in activity recording mode*/
						/* or pulse amplitude in pulse mode */
} probe_descriptor_struct;

#endif
